﻿package com.Islands 
{
	import adobe.utils.CustomActions;
	import org.flixel.*;
	import util.ExtraFlxTileMap;

	public class GameState extends FlxState
	{
		//MAPS
		[Embed(source = '../../gfx/Island Tileset.png')] private var IslandTiles:Class;
		[Embed(source = '../../gfx/Signpost.png')] private var ImgIslandSignposts:Class;

		[Embed(source = "../../gfx/Island - level 1.txt", mimeType = "application/octet-stream")] public static var data_Island_map:Class;
		[Embed(source = "../../gfx/Island - signposts level 1.txt", mimeType = "application/octet-stream")] public static var data_Island_signposts:Class;
		[Embed(source = "../../gfx/Island - level 1.xml", mimeType = "application/octet-stream")] public static var data_Island_objects:Class;
		[Embed(source = "../../gfx/Island - water level 1.txt", mimeType = "application/octet-stream")] public static var data_Island_water:Class;
		[Embed(source = "../../gfx/Island - cloneblock level 1.txt", mimeType = "application/octet-stream")] public static var data_Island_cloneblock:Class;
		protected var _islandmap:FlxTilemap;
		protected var _islandsignposts:ExtraFlxTileMap;
		protected var _islandswater:FlxTilemap;
		protected var _islandscloneblock:FlxTilemap;
		
		protected var cheatar:Boolean = false;
		
		
		protected var _switch_group:FlxGroup;
		protected var _psi_group:FlxGroup;
		protected var _checkpoint:Checkpoint;
		
		//PLAYER
		protected var _player:Player;
		protected var _clone:PlayerClone;
		
		protected var _background:Background;
		
		//enemies
		protected var _enemy_group:FlxGroup;
		protected var _text_sign_group:FlxGroup;
		
		protected var _time_elapsed:Number;
		protected var _target_recede_time:Number;
		protected var _starting_recede_y:Number;
		protected const _total_recede_time:Number = 4;
		protected const _total_recede_tile_height:Number = 4;
		
		public function GameState() 
		{
			super();
			bgColor = 0xff34C2ED;
			_time_elapsed = 0;
		}
		
		public function Win():void
		{
			SoundLibrary.StopSound(SoundLibrary.SOUND_MUSIC_THEME, 2);
			SoundLibrary.PlaySound(SoundLibrary.SOUND_WIN);
			FlxG.fade.start(0xffffffff, 2, ChangeToWinState, true);
			
		}
		
		protected function ChangeToWinState():void
		{
			// change state.
			FlxG.state = new WinState();
		}
		
		public function Spawn():void
		{
			//if (_checkpoint.visible)
			_player.reset(_checkpoint.x, _checkpoint.y);
			//else
			//	_player.reset(Islands.TILE_SIZE * 105, Islands.TILE_SIZE * 27);
		}
		
		public function WaterRecede():void
		{
			//reduce water level!!!!
			_target_recede_time = _time_elapsed + _total_recede_time;
			_starting_recede_y = _islandswater.y;
		}
		
		protected function LoadMapAndObjects():void
		{
			_background = new Background();
			this.add(_background);
			
			_islandmap = new FlxTilemap();
			_islandmap.loadMap(new data_Island_map, IslandTiles, Islands.TILE_SIZE);
			_islandmap.collideIndex = 9;
			this.add(_islandmap);
			
			_islandsignposts = new ExtraFlxTileMap();
			_islandsignposts.loadMap(new data_Island_signposts, ImgIslandSignposts, Islands.TILE_SIZE);
			this.add(_islandsignposts);
			
			_checkpoint = new Checkpoint(Islands.TILE_SIZE * 105, Islands.TILE_SIZE * 27);
			this.add(_checkpoint);
			
			_enemy_group = new FlxGroup();
			var objs:XML = XML(new data_Island_objects);
			
			var list :XML = objs.objects[0];
			var o:XML;
			for each (o in list.OBJ1)
			{
				_enemy_group.add(new Enemy(o.@x * Islands.TILE_SIZE, o.@y* Islands.TILE_SIZE));
			}
			
			_text_sign_group = new FlxGroup();
			for each (o in list.OBJ11)
			{
				_text_sign_group.add(new TextSign(o.@x * Islands.TILE_SIZE, o.@y * Islands.TILE_SIZE,
					"Use arrow keys to move, and X to jump. Water is dangerous."));
			}
			
			for each (o in list.OBJ12)
			{
				_text_sign_group.add(new TextSign(o.@x * Islands.TILE_SIZE, o.@y * Islands.TILE_SIZE,
					"Enemies will hurt you and send you back to the last checkpoint."));
			}
			
			for each (o in list.OBJ13)
			{
				_text_sign_group.add(new TextSign(o.@x * Islands.TILE_SIZE, o.@y * Islands.TILE_SIZE,
					"Bridge the gap between you and your foe by pressing the C key."));
			}
			
			for each (o in list.OBJ14)
			{
				_text_sign_group.add(new TextSign(o.@x * Islands.TILE_SIZE, o.@y * Islands.TILE_SIZE,
					"Islands are only separated by water."));
			}
			
			for each (o in list.OBJ15)
			{
				_text_sign_group.add(new TextSign(o.@x * Islands.TILE_SIZE, o.@y * Islands.TILE_SIZE,
					"It looks like the water level has receded. New islands might be within reach."));
			}
			
			//add switches
			_switch_group = new FlxGroup();
			for each (o in list.OBJ20)
			{
				_switch_group.add(new WaterSwitch(o.@x * Islands.TILE_SIZE, o.@y * Islands.TILE_SIZE));
			}
			this.add(_switch_group);
			
			//add winning Psi
			_psi_group = new FlxGroup();
			for each (o in list.OBJ100)
			{
				_psi_group.add(new Psi(o.@x * Islands.TILE_SIZE, o.@y * Islands.TILE_SIZE));
			}
			
			this.add(_psi_group);
			
			
			this.add(_enemy_group);
			
			_islandscloneblock = new FlxTilemap();
			_islandscloneblock.loadMap(new data_Island_cloneblock, IslandTiles, Islands.TILE_SIZE);
			if(!cheatar)
				this.add(_islandscloneblock);
			
			//water is last, it covers everything.
			_islandswater = new FlxTilemap();
			_islandswater.loadMap(new data_Island_water, IslandTiles, Islands.TILE_SIZE);
			this.add(_islandswater);
			
			
			
			//topmost
			this.add(_text_sign_group);
		}
		
		override public function create():void 
		{
			super.create();
			
			SoundLibrary.PlaySound(SoundLibrary.SOUND_MUSIC_THEME, 10);
			bgColor = 0xffffffff;
			_target_recede_time = 0;
			
			LoadMapAndObjects();
			
			_player = new Player();
			_clone = new PlayerClone(_player);
			Spawn();
			this.add(_player);
			this.add(_clone);
			
			FlxG.follow(_player, Islands.LERP);
			_islandmap.follow();
		}
		
		protected function EaseInOutCurve(input:Number):Number
		{
			return Math.pow(Math.sin(input * Math.PI / 2), 2);
		}
		
		public function IsWaterSet(Contact:FlxObject):Boolean
		{
			return (Contact == _islandswater);
		}
		
		override public function update():void 
		{
			super.update();
			SoundLibrary.Update();
			
			if (_target_recede_time > _time_elapsed)
			{
				_islandswater.y = _starting_recede_y +
					_total_recede_tile_height * Islands.TILE_SIZE
					* EaseInOutCurve((1 - ((_target_recede_time - _time_elapsed) / _total_recede_time)));
			}
			
			_time_elapsed += FlxG.elapsed;
			_islandmap.collide(_player);
			_islandmap.collide(_enemy_group);
			_islandswater.collide(_player);
			_islandswater.collide(_clone);
			
			_psi_group.collide(_player);
			
			if(!cheatar)
				_islandscloneblock.collide(_clone);
			
			_switch_group.collide(_player);
			
			//(do not collide island with player)
			
			_enemy_group.collide(_player);
			_enemy_group.collide(_clone);
			
			if (_islandsignposts.overlaps(_player))
			{
				HandleSignpostOverlap();
			}
			
			//kind of messy to put this here, but I don't mind it all that much.
			if (FlxG.keys.justPressed("C"))
			{
				//deal with new clone
				if (!_clone.exists)
				{
					if (_player.exists)
					{
						_clone.setclone();
					}
				}
				
			}
			
			if (FlxG.keys.justPressed("R"))
			{
				//if (FlxG.debug)
				{
					//respawn
					if (_player.exists)
					{
						Spawn();
					}
				}
			}
			
			if (FlxG.keys.justPressed("B"))
			{
				if (FlxG.debug)
					FlxG.showBounds = !FlxG.showBounds;
			}
		}
		
		protected function HandleSignpostOverlap():void
		{
			var overlap:FlxObject =  _islandsignposts.get_overlap_tile(_player);
			_checkpoint.Set(overlap.x, overlap.y);
			
			for each(var ob:FlxObject in _text_sign_group.members)
			{
				if (ob.overlaps(_player))
				{
					(ob as TextSign).Show();
				}
			}
		}
	}
}